from rocolib.api.components import FoldedComponent
from rocolib.api.composables.graph.Face import Face
from rocolib.utils.numsym import cumsum


def isclose(a, b, rel_tol=1e-09, abs_tol=0.0):
    return abs(a-b) <= max(rel_tol * max(abs(a), abs(b)), abs_tol)

class SplitEdge(FoldedComponent):
    def define(self):
        self.addParameter("toplength", (50, 50), valueType="(tuple, list)")
        self.addParameter("botlength", (100,), valueType="(tuple, list)")
        self.addParameter("width", 0)

        for i in range(100):
          self.addEdgeInterface("topedge%d" % i, None, "toplength")
          self.addEdgeInterface("botedge%d" % i, None, "botlength")

    def assemble(self):
        t = cumsum(self.getParameter("toplength")[::-1])
        b = cumsum(self.getParameter("botlength")[::-1])
        if not isclose(t[-1], b[-1]):
          raise ValueError("SplitEdge lengths not equal: %s <> %s" % (repr(t), repr(b)))

        w = self.getParameter("width")
        pts = [(x, 0) for x in b]
        pts += [(x, w) for x in t[::-1]]
        pts += [(0, w), (0,0)]

        self.addFace(Face("split", pts))

        tops = ["e%d" % (len(b) + d + 1) for d in range(len(t))]
        bots = ["e%d" % d for d in range(len(b))[::-1]]
        for i, topedge in enumerate(tops):
          self.setEdgeInterface("topedge%d" % i, topedge, "toplength")
        for i, botedge in enumerate(bots):
          self.setEdgeInterface("botedge%d" % i, botedge, "botlength")

if __name__ == "__main__":
    SplitEdge.test()

